####
__title__ = 'icon-getinfo'
__description__ = 'Icon Blockchain Network node get information.'
__url__ = 'https://github.com/HonamSong/icon_getinfo'
__version__ = '0.0.9'
__author__ = 'Honam Song'
__author_email__ = 'hnsong@iconloop.com'
__license__ = 'MIT'
__copyright__ = 'Copyright 2022 hnsong'
